/* +++Date last modified: 05-Jul-1997 */

/* =======================================================================
    CFG.c       Generic configuration file handler.

                A. Reitsma, Delft, The Netherlands.
                v1.00  94-07-09  Public Domain.

                Modifyied 03.04.99 by Victor Pomortseff
                2:5080/111.26@fidonet <root@pomor.e-burg.ru>
 ---------------------------------------------------------------------- */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include "cfg.h"

#define LINE_LEN_MAX    4096                 /* actual max line length  */
#define BUFFERSIZE      LINE_LEN_MAX +2      /* ... including \n and \0 */

enum RetVal
{
    NO_PROBLEMS,
    ERR_FOPEN,
    ERR_MEM,
};

struct CfgStrings *NewItem()
{
  struct CfgStrings *Item;
  
  Item = (struct CfgStrings*)malloc(sizeof(struct CfgStrings));
  if(Item != NULL)
  {
    Item->Next = NULL;
    Item->name = NULL;
    Item->data = NULL;
  }
  
  return Item;
}

void CfgFree(struct CfgStrings *CfgInfo)
{
  struct CfgStrings *Item;
  
  while(CfgInfo != NULL)
  {
    if(CfgInfo->name != NULL) free(CfgInfo->name);
    if(CfgInfo->data != NULL) free(CfgInfo->data);
    Item = CfgInfo;
    CfgInfo = CfgInfo->Next;
    free(Item);
  };
}

int CfgRead(char *Filename, struct CfgStrings **CfgInfo)
{
    char Buffer[ BUFFERSIZE ];
    char * WorkPtr ;
    char * CfgName ;
    char * CfgData ;
    struct CfgStrings * Cfg ;
    FILE * CfgFile ;

    *CfgInfo = NULL;
    CfgFile = fopen( Filename, "r" );
    if(NULL == CfgFile)
    {
        return ERR_FOPEN ;
    }

    while(NULL != fgets(Buffer, BUFFERSIZE, CfgFile))
    {
        /* clip off optional comment tail indicated by a semi-colon
        */
        /*strlwr(Buffer);*/
        if( NULL != (WorkPtr = strchr(Buffer, ';')))
            *WorkPtr = '\0';
        else
            WorkPtr = Buffer + strlen(Buffer);

        /* clip off trailing and leading white space
        */
        WorkPtr--;
        while(isspace( *WorkPtr ) && WorkPtr >= Buffer)
            *WorkPtr-- = '\0';
        WorkPtr = Buffer;
        while( isspace(*WorkPtr))
            WorkPtr++;
        if( 0 == strlen(WorkPtr))
            continue;

        CfgName = strtok(WorkPtr, " =");
        if( NULL != CfgName )
        {
            /* Condition the name (lower case required),
               and strip leading white and a 'late' = from data part.
            */
            CfgData = strtok( NULL, "" );
            if(CfgData != NULL)
            {
              while( isspace( *CfgData )) CfgData++;
              if( '=' == *CfgData ) CfgData++;
              while( isspace( *CfgData )) CfgData++;
            }
            /* add new pair 'name'-'data'
            */
            if(*CfgInfo == NULL)
            {
              *CfgInfo = NewItem();
              Cfg = *CfgInfo;
            }
            else
            {
              Cfg->Next = NewItem();
              Cfg = Cfg->Next;
            }

            /* duplicate the data if the name is found.
            */
            if(NULL != Cfg)
            {
                if(CfgName != NULL) Cfg->name = strdup(strlwr(CfgName));
                else Cfg->name = strdup("");

                if(CfgData != NULL) Cfg->data = strdup(CfgData); /* strdup is not ANSI    */
                else Cfg->data = strdup("");                     /* memory leaks if Cfg->data */
                                                                 /* is malloc'ed already      */
                if(NULL == Cfg->data || NULL == Cfg->name)
                {
                    fclose( CfgFile );
                    CfgFree(*CfgInfo);
                    *CfgInfo = NULL;
                    return ERR_MEM;
                }
            }   /* undetected error on failure should not be a problem  */
                /* as configuration reading should be done early.       */
        }       /* but test and handle it anyway ...                    */
    }
    fclose(CfgFile);
    return NO_PROBLEMS ;
}


#ifdef TEST

struct CfgStrings *CfgDemoData;

main( int argc, char * argv[] )
{
    int ix ;
    struct CfgStrings *Item;

    if( argc > 1 )
        CfgRead( argv[1], &CfgDemoData);
    else
        return ERR_FOPEN ;

    printf( "%s\n", argv[1]);
    Item = CfgDemoData;
    while(Item != NULL)
    {
       printf( "CfgItem \"%s\" is \"%s\"\n", Item->name,
                                             Item->data);
       Item = Item->Next;
    };
    CfgFree(CfgDemoData);
    
    return NO_PROBLEMS;
}
#endif

/* ==== CFG.c end ===================================================== */
