#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <process.h>
#include "cfg.h"
#include "ml2news.h"

tagFIELDS2REMOVE FieldsToRemove[] = {{"From", FIELD_FROM},
                                     {"Received:", FIELD_RECEIVED},
                                     {"To:", FIELD_TO},
                                     {"Precedence:", FIELD_PRECEDENCE},
                                     {"Cc:", FIELD_CC},
                                     {"Subject:", FIELD_SUBJECT}
                                    };
int nFields                       = sizeof(FieldsToRemove) / sizeof(FieldsToRemove[0]);
tagLISTSIDLIST *pLists            = NULL;
int ClearSubj                     = 0;
char *Rel2Fido                    = NULL;

int cfgparce(char *cfgname)
{
  int rc = 0;
  CfgStrings *pConfig = NULL;
  CfgStrings *pCurrent;
  
  if(cfgname != NULL)
  {
    if(!CfgRead(cfgname, &pConfig))
    {
      for(pCurrent = pConfig; pCurrent != NULL; pCurrent = pCurrent->Next)
      {
        if(!stricmp(pCurrent->name, "List"))
        {
          static char tmpfld1[128], tmpfld2[128], tmpfld3[128];
          
          if(sscanf(pCurrent->data, "%s %s %s", tmpfld1, tmpfld2, tmpfld3) == 3)
          {
            tagLISTSIDLIST *pNewList = (tagLISTSIDLIST*)malloc(sizeof(tagLISTSIDLIST));

            if(pNewList != NULL)
            {
              pNewList->pNext = pLists;
              pNewList->flen = strlen(tmpfld2);
              pNewList->pKey = strdup(tmpfld1);
              pNewList->pField = strdup(tmpfld2);
              pNewList->pName = strdup(tmpfld3);
              
              pLists = pNewList;
            }
          }
        }
        else if(!stricmp(pCurrent->name, "ClearSubject"))
        {
          if(!stricmp(pCurrent->data, "Yes"))
            ClearSubj = 1;
        }
        else if(!stricmp(pCurrent->name, "Rel2Fido"))
        {
          Rel2Fido = strdup(pCurrent->data);
        }
      }
      
      CfgFree(pConfig);
      rc = 1;
    }
  }
  
  return rc;
}

void procstring(char *str, FILE *stream)
{
  static int lastfield = FIELD_OTHER;
  static char newsgroup[1024];
  static int newsready = 0;
  tagLISTSIDLIST *pCurList = pLists;
  int curfield;
  int fieldlen;

  if(isspace(str[0]))
    curfield = lastfield;
  else
  {
    curfield = lastfield = FIELD_OTHER;
    
    for(int i = 0; i < nFields; i++)
    {
      fieldlen = strlen(FieldsToRemove[i].pName);
      
      if(!strncmp(str, FieldsToRemove[i].pName, fieldlen) && 
         isspace(str[fieldlen]))
      {
        curfield = lastfield = FieldsToRemove[i].iCode;
        break;
      }
    }
  }
  
  if(!newsready)
  {
    for(pCurList = pLists; pCurList != NULL; pCurList = pCurList->pNext)
    {
      if(!strncmp(str, pCurList->pField, pCurList->flen) &&
         isspace(str[pCurList->flen]))
      {
        char *p = &str[pCurList->flen + 1];

        if(strstr(p, pCurList->pKey) != NULL)
        {
          sprintf(newsgroup, "Newsgroups: %s\n", pCurList->pName);
          fputs(newsgroup, stream);  
          newsready = 1;
          break;
        }
      }
    }
  }
  
  if(curfield == FIELD_SUBJECT)
  {
    if(ClearSubj)
    {
      char *p = &str[fieldlen + 1];
      
      if(*p == '[')
      {
        char *q = strchr(p, ']');
        
        if(q)
        {
          q++;
          if(isspace(*q)) q++;
          memmove(p, q, (strlen(q) + 1));
        }
      }
    }
  }
  
  if(curfield == FIELD_OTHER || 
     curfield == FIELD_SUBJECT)
    fputs(str, stream);
}

void procfile(char *fname)
{
  FILE *in = stdin;
  FILE *out;
  char *tmpfname = TEMPORARY_FILE_NAME;
  static char str[8196];

  if(fname != NULL)
  {
    in = fopen(fname, "rb");
    
    if(in == NULL)  
    {
      fprintf(stderr, "Couldn't open input file '%s'\n", fname);
      return;
    }
  }
  
  if(tmpfname != NULL)
  {
    out = fopen(tmpfname, "wb");

    if(out == NULL)  
    {
      fprintf(stderr, "Couldn't open temporary output file '%s'\n", tmpfname);
      return;
    }
  }
  else
  {
    fprintf(stderr, "Couldn't create temporary file name\n");
    return;
  }
  
  do
  {
    fgets(str, 8195, in);
    procstring(str, out);
  } while(!feof(in));
  
  if(fname != NULL)
    fclose(in);

  fclose(out);
  
  if(Rel2Fido == NULL)
    sprintf(str, "rel2fido -r < %s", tmpfname);
  else
    sprintf(str, "%s -r < %s", Rel2Fido, tmpfname);

  system(str);
  unlink(tmpfname);
}

void main(int argc, char *argv[])
{
  static char cfg_path[_MAX_PATH];
  static char drive[_MAX_DRIVE];
  static char dir[_MAX_DIR];
  static char fname[_MAX_FNAME];
  static char ext[_MAX_EXT];

  _splitpath(argv[0], drive, dir, fname, ext);
  _makepath(cfg_path, drive, dir, fname, "cfg");

  if(cfgparce(cfg_path))
  {
    if(argc > 1)
      procfile(argv[1]);
    else
      procfile(NULL);
  }
  
  while(pLists != NULL)
  {
    tagLISTSIDLIST *pCurList = pLists;
    
    pLists = pLists->pNext;
    
    if(pCurList->pKey)
      free(pCurList->pKey);
    
    if(pCurList->pField)
      free(pCurList->pField);
      
    if(pCurList->pName)
      free(pCurList->pName);
      
    free(pCurList);
  };
  
  if(Rel2Fido != NULL)
    free(Rel2Fido);
    
  exit(0);  
}


